import re
from os import path
from typing import Type, Union, Dict, Any, Optional

from diagrams import Node
from diagrams.generic.compute import Rack
from dockerfile_parse import DockerfileParser

from docker_compose_diagram.docker_images.patterns import DockerImagePattern

DEFAULT_ICON_CLASS = Rack


def read_dockerfile_image(service_info):
    build = service_info.get('build', {})
    context = build.get('context')
    if context is None:
        return None

    dockerfile_path = build.get('dockerfile')
    if dockerfile_path is None:
        dockerfile_path = path.join(context, 'Dockerfile')
    else:
        dockerfile_path = path.join(context, dockerfile_path)

    dfp = DockerfileParser()
    with open(dockerfile_path, 'r') as file:
        dfp.content = file.read()

    return dfp.baseimage


def determine_image_name(
        service_info: Dict[str, Any],
) -> Optional[str]:
    image_name = service_info.get('image')
    if image_name is None:
        image_name = read_dockerfile_image(service_info=service_info)

    return image_name


def determine_diagram_render_class(
        image_name: str,
) -> Union[Type[DockerImagePattern], Type[Node]]:
    if image_name is None:
        return DEFAULT_ICON_CLASS

    for subclass in DockerImagePattern.__subclasses__():
        re_match = re.search(subclass.pattern, image_name)
        if re_match:
            return subclass.diagram_render_class

    return DEFAULT_ICON_CLASS



from diagrams.aws.database import Dynamodb
from diagrams.aws.integration import SNS, SQS
from diagrams.aws.storage import S3
from diagrams.generic.compute import Rack
from diagrams.onprem.database import PostgreSQL, MySQL, Mongodb
from diagrams.onprem.inmemory import Redis
from diagrams.onprem.network import Nginx, Traefik, Caddy
from diagrams.onprem.queue import Celery, Kafka
from diagrams.onprem.queue import RabbitMQ
from diagrams.programming.framework import Django, Flask, Rails, React, Vue
from diagrams.programming.language import (
    Python,
    Ruby,
    Go,
    Rust,
    Kotlin,
    Javascript,
    NodeJS,
)

DEFAULT_ICON_CLASS = Rack


class DockerImagePattern:
    pattern = r''
    diagram_render_class = None

    def render(self, service_name):
        return self.diagram_render_class(service_name)


class RedisImage(DockerImagePattern):
    pattern = r'redis:'
    diagram_render_class = Redis


class PostgreSQLImage(DockerImagePattern):
    pattern = r'postgres:'
    diagram_render_class = PostgreSQL


class NginxImage(DockerImagePattern):
    pattern = r'nginx'
    diagram_render_class = Nginx


class PythonImage(DockerImagePattern):
    pattern = r'python'
    diagram_render_class = Python


class MysqlImage(DockerImagePattern):
    pattern = r'mysql'
    diagram_render_class = MySQL


class SNSImage(DockerImagePattern):
    pattern = r'sns$'
    diagram_render_class = SNS


class SQSImage(DockerImagePattern):
    pattern = r'sqs'
    diagram_render_class = SQS


class DynamoDBImage(DockerImagePattern):
    pattern = r'dynamodb'
    diagram_render_class = Dynamodb


class RubbyImage(DockerImagePattern):
    pattern = r'ruby'
    diagram_render_class = Ruby


class GolangImage(DockerImagePattern):
    pattern = r'golang'
    diagram_render_class = Go


class RabbitMQImage(DockerImagePattern):
    pattern = r'rabbitmq'
    diagram_render_class = RabbitMQ


class DjangoImage(DockerImagePattern):
    pattern = r'django'
    diagram_render_class = Django


class CeleryImage(DockerImagePattern):
    pattern = r'celery'
    diagram_render_class = Celery


class FlaskImage(DockerImagePattern):
    pattern = r'flask'
    diagram_render_class = Flask


class S3Image(DockerImagePattern):
    pattern = r's3'
    diagram_render_class = S3


class RailsImage(DockerImagePattern):
    pattern = r'rails'
    diagram_render_class = Rails


class RustImage(DockerImagePattern):
    pattern = r'rust'
    diagram_render_class = Rust


class KotlinImage(DockerImagePattern):
    pattern = r'kotlin'
    diagram_render_class = Kotlin


class NodeJSImage(DockerImagePattern):
    pattern = r'node'
    diagram_render_class = NodeJS


class JavaScriptImage(DockerImagePattern):
    pattern = r'javascript'
    diagram_render_class = Javascript


class MongodbImage(DockerImagePattern):
    pattern = r'mongo'
    diagram_render_class = Mongodb


class ReactImage(DockerImagePattern):
    pattern = r'react'
    diagram_render_class = React


class VueImage(DockerImagePattern):
    pattern = r'vue'
    diagram_render_class = Vue


class KafkaImage(DockerImagePattern):
    pattern = r'kafka'
    diagram_render_class = Kafka


class TraefikImage(DockerImagePattern):
    pattern = r'traefik'
    diagram_render_class = Traefik


class CaddyImage(DockerImagePattern):
    pattern = r'caddy'
    diagram_render_class = Caddy



from __future__ import annotations

from dataclasses import dataclass, field
from collections import defaultdict
from typing import List, Dict, Type, Any, Optional

from diagrams import Node

from docker_compose_diagram.docker_images.utils import determine_diagram_render_class

LABELS_PREFIX = 'docker_compose_diagram'
CLUSTER_LABEL_VALUE = f'{LABELS_PREFIX}.cluster'
ICON_LABEL_VALUE = f'{LABELS_PREFIX}.icon'
DESCRIPTION_LABEL_VALUE = f'{LABELS_PREFIX}.description'


@dataclass
class DockerComposeService:
    name: str
    service_info: Dict[str, Any]

    cluster: Optional[str] = None
    description: str = ''
    diagram_render_class: Type[Node] = None
    diagram_render_class_instance: Node = None

    def __post_init__(self):
        labels = self.service_info.get('labels', {})
        self.cluster = labels.get(CLUSTER_LABEL_VALUE)
        self.description = labels.get(DESCRIPTION_LABEL_VALUE, '')

        label_icon = labels.get(ICON_LABEL_VALUE)
        if isinstance(label_icon, str):
            self.diagram_render_class = determine_diagram_render_class(
                image_name=label_icon,
            )

    def render(self):
        self.diagram_render_class_instance = self.diagram_render_class(
            self.name +
            f'\n{self.description}',
        )
        return self.diagram_render_class_instance

    def render_dependency_arrows(self, storage: DockerComposeServiceStorage) -> None:
        for dependency_name in self.service_info.get('depends_on', []):
            dependency_service = storage.find_by_name(name=dependency_name)
            self.render_dependency_arrow(dependency=dependency_service)

    def render_dependency_arrow(self, dependency: DockerComposeService) -> None:
        return self.diagram_render_class_instance >> dependency.diagram_render_class_instance


@dataclass
class DockerComposeServiceStorage:
    docker_compose_services: Dict[str, DockerComposeService] = field(
        default_factory=dict,
    )

    def add(self, service: DockerComposeService) -> None:
        if service.name not in self.docker_compose_services.keys():
            self.docker_compose_services[service.name] = service

    def all(self) -> List[DockerComposeService]:
        return list(self.docker_compose_services.values())


    def group_by_cluster(self) -> Dict[str, List[DockerComposeService]]:
        cluster_name_to_services = defaultdict(list)
        for service in self.docker_compose_services.values():
            cluster_name_to_services[service.cluster].append(service)

        return cluster_name_to_services

    def find_by_name(self, name: str) -> DockerComposeService:
        return self.docker_compose_services[name]



import sys
from typing import Dict, Any

import yaml
from diagrams import Cluster, Diagram

from docker_compose_diagram.docker_compose.services import DockerComposeServiceStorage, DockerComposeService
from docker_compose_diagram.docker_images.utils import determine_diagram_render_class, determine_image_name


def collect_services_into_storage(
        docker_compose_parsed: Dict[str, Any],
) -> DockerComposeServiceStorage:
    diagram_nodes_storage = DockerComposeServiceStorage()
    for service_name, service_info in docker_compose_parsed["services"].items():
        image_name = determine_image_name(
            service_info=service_info,
        )
        render_class = determine_diagram_render_class(
            image_name=image_name,
        )

        docker_compose_service = DockerComposeService(
            name=service_name,
            service_info=service_info,
            diagram_render_class=render_class,
        )
        diagram_nodes_storage.add(service=docker_compose_service)

    return diagram_nodes_storage


def render_clustered_services(storage: DockerComposeServiceStorage) -> None:
    clusters: Dict = storage.group_by_cluster()
    for cluster_name, cluster_services in clusters.items():
        if cluster_name is None:
            continue

        with Cluster(cluster_name):
            for service in cluster_services:
                service.render()


def render_not_clustered_services(storage: DockerComposeServiceStorage) -> None:
    clusters: Dict = storage.group_by_cluster()
    not_clustered_services = clusters.pop(None, [])

    for service in not_clustered_services:
        service.render()


def render_dependencies(storage: DockerComposeServiceStorage):
    for service in storage.all():
        service.render_dependency_arrows(storage=storage)


def pars_yaml_file(file_path: str) -> Dict[str, Any]:
    with open(file_path, 'r') as stream:
        try:
            parsed_yaml_structure = yaml.safe_load(stream)
        except yaml.YAMLError:
            print('Failed to parse file')
            sys.exit()

    return parsed_yaml_structure


def draw(file, direction, graph_attr):
    docker_compose_parsed = pars_yaml_file(file_path=file)
    diagram_nodes_storage = collect_services_into_storage(docker_compose_parsed)

    with Diagram("docker-compose", show=False, direction=direction, graph_attr=graph_attr):
        with Cluster(file):
            render_clustered_services(storage=diagram_nodes_storage)
            render_not_clustered_services(storage=diagram_nodes_storage)
            render_dependencies(storage=diagram_nodes_storage)



import click

from docker_compose_diagram.draw import draw


@click.command()
@click.option('--file', default='docker-compose.yml', help='docker-compose file')
@click.option('--direction', default="TB", type=click.Choice(["TB", "BT", "LR", "RL"], case_sensitive=True))
@click.option('--nodesep', default="1.0", type=click.FLOAT)
def process_cli(file, direction, nodesep):
    graph_attr = {
        "nodesep": str(nodesep),
    }
    draw(file, direction, graph_attr)


if __name__ == '__main__':
    process_cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['docker_compose_diagram',
 'docker_compose_diagram.docker_compose',
 'docker_compose_diagram.docker_images']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML==5.3.1',
 'click>=8.0.3,<9.0.0',
 'diagrams>=0.20.0,<0.21.0',
 'dockerfile-parse>=1.2.0,<2.0.0',
 'rich>=12.2.0,<13.0.0']

entry_points = \
{'console_scripts': ['compose-diagram = docker_compose_diagram:process_cli']}

setup_kwargs = {
    'name': 'docker-compose-diagram',
    'version': '0.2.0',
    'description': '',
    'long_description': None,
    'author': 'Skonik',
    'author_email': 's.konik.dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
